#!/usr/bin/perl -w

# Take the tab separated outptu of ec2-decribe-group and emits a bash script
# which can be run to recreate input.  You can update boilerplate at the bottom
# of this script.
#
# Example usage:
#   ec2-describe-group | ./create-firewall-script.pl > create-firewall.sh

use strict;
use diagnostics;

my %networks = qw(
    1.2.3.4/32  OFFICE
    2.3.4.5/32   CACTI
    );

my %other_vars= qw(
    us-west-1    ZONE
    342805292197 ACCT
    );

print <DATA>;  # print the header
print "# Autogenerated at " . localtime() . "\n\n";

print "# networks\n";
foreach my $cidr (keys %networks) {
    print $networks{$cidr}, "=", $cidr, "\n";
}

print "\n# settings\n";
foreach my $val (keys %other_vars) {
    print $other_vars{$val}, "=", $val, "\n";
}
print "\n";

while (<>) {
    chomp;
    my ($type, @cols) = split "\t";

    if ($type eq "GROUP") {
        &add_group(@cols);
    } elsif ($type eq "PERMISSION") {
        &add_authorize(@cols);
    } else {
        die "Unrecognized type: $type";
    }
}

sub add_group {
    my ($id, $owner, $name, $desc) = @_;
    print "\nec2-create-group --region \$ZONE $name --description \"$desc\"\n";
}

sub add_authorize {
    my ($owner, $name, $type, $protocol, $port_start, $port_end, $from_to,
        $principle_type) = @_;

    print "ec2-authorize --region \$ZONE $name";

    if ($protocol eq 'icmp') {
        print " --protocol $protocol -t $port_start:$port_end"
    } elsif ($protocol eq 'tcp' || $protocol eq 'udp') {
        print " --protocol $protocol";
        print " -p $port_start";
        if ($port_start != $port_end) {
            print "-$port_end";
        }
    } else {
        die "Unrecognizd protocol '$protocol'\n";
    }

    if ($principle_type eq "USER") {
        &add_authorize_group(@_);
    } elsif ($principle_type eq "CIDR") {
        &add_authorize_cidr(@_);
    } else {
        die "Unrecognized permission type: $principle_type\n";
    }

    print "\n";
}

sub add_authorize_group {
    my ($user, $name_pair) = @_[8,9];
    my $group;
    (($group) = ($name_pair =~ /^NAME\s+(\S+)$/))
        or die "Bad group name '$name_pair'\n";

    $user = '$' . $other_vars{$user} if ($other_vars{$user});

    print " -u $user -o $group";
}

sub add_authorize_cidr {
    my ($cidr) = $_[8];

    if ($networks{$cidr}) {
        print " --cidr \$", $networks{$cidr};
    } elsif ($cidr ne '0.0.0.0/0') { # skip if default
        print " --cidr $cidr";
    }
}

# stuff below here becomes the new script header
__DATA__
#!/bin/sh
#
# Create EC2 security groups.  Don't forget to set your EC2_HOME and
# EC2_PRIVATE_KEY env variables.  And update ZONE in the script if you're
# creating in a different availability zone.
#
